@@ -65,7 +65,6 @@ class AbstractEndpointValidator:
     def __iter__(self):
         yield "id", self.id
         yield "name", self.name
-        yield "type", self.type
         yield "specialId", self.special_id
         yield "constraints", None
 
@@ -102,7 +101,15 @@ class BooleanEndpointValidator(AbstractEndpointValidator):
         # check that data is a boolean value
         if not isinstance(data, bool):
             try:
-                data = bool(data)
+                if isinstance(data, str):
+                    if data.lower() == "true":
+                        data = True
+                    elif data.lower() == "false":
+                        data = False
+                    else:
+                        return ValidationResponse.INVALID_TYPE, None
+                else:
+                    data = bool(data)
             except ValueError:
                 return ValidationResponse.INVALID_TYPE, None
 
@@ -111,7 +118,7 @@ class BooleanEndpointValidator(AbstractEndpointValidator):
     def __iter__(self):
         yield "id", self.id
         yield "name", self.name
-        yield "type", self.type
+        yield "type", self.type.value
         yield "specialId", self.special_id
         yield "constraints", None
 
@@ -166,7 +173,7 @@ class IntegerEndpointValidator(AbstractEndpointValidator):
     def __iter__(self):
         yield "id", self.id
         yield "name", self.name
-        yield "type", self.type
+        yield "type", self.type.value
         yield "specialId", self.special_id
         yield "constraints", {
             "min": self.min,
@@ -267,7 +274,7 @@ class StringEndpointValidator(AbstractEndpointValidator):
     def __iter__(self):
         yield "id", self.id
         yield "name", self.name
-        yield "type", self.type
+        yield "type", self.type.value
         yield "specialId", self.special_id
         yield "constraints", {
             "length": self.length,
@@ -365,7 +372,7 @@ class EnumEndpointValidator(AbstractEndpointValidator):
     def __iter__(self):
         yield "id", self.id
         yield "name", self.name
-        yield "type", self.type
+        yield "type", self.type.value
         yield "specialId", self.special_id
         yield "constraints", {
             "values": self.values
