@@ -345,7 +345,14 @@ class IoTManager(object):
         client = self.__clients.get(client_id, None)
 
         if client is None:
-            return ValidationResponse.CLIENT_NOT_CONNECTED
+            validator = self.on_get_offline_validator(client_id, endpoint_id)
+
+            if isinstance(validator, ValidationResponse):
+                return validator
+            elif isinstance(validator, AbstractEndpointValidator):
+                return validator
+            else:
+                return ValidationResponse.CLIENT_NOT_CONNECTED
         else:
             return client.get_validator(endpoint_id)
 
@@ -481,12 +488,14 @@ class IoTManager(object):
         :return:
         """
         if client_id is not None:
+            self.logger.info("emitting event '" + event + "' to client of ID '" + client_id + "'")
             client = self.__clients.get(client_id, None)
 
             if client:
                 client.emit(event, data, trigger_update)
         elif room is not None:
             if self.__rooms.get(room, None) is not None:
+                self.logger.info("emitting event '" + event + "' to clients in room '" + room + "'")
                 for client_id in self.__rooms[room]:
                     client = self.__clients.get(client_id, None)
 
@@ -497,6 +506,7 @@ class IoTManager(object):
                             if client.type == client_type:
                                 client.emit(event, data, trigger_update)
         elif client_type is not None:
+            self.logger.info("emitting event '" + event + "' to clients of type '" + client_type + "'")
             for client in self.__clients.values():
                 if client.type == client_type:
                     client.emit(event, data, trigger_update)
@@ -526,8 +536,10 @@ class IoTManager(object):
             self.__clients[client].join(room, called_by_manager=True)
 
         if self.__rooms.get(room, None) is None:
+            self.logger.info("adding client of ID '" + client + "' to newly created room '" + room + "'")
             self.__rooms[room] = [client]
         else:
+            self.logger.info("adding client of ID '" + client + "' to existing room '" + room + "'")
             self.__rooms[room].append(client)
 
     def leave(self, client: Union[IoTClient, str], room: str, **kwargs):
@@ -555,6 +567,7 @@ class IoTManager(object):
         if self.__rooms.get(room, None) is None:
             return
         else:
+            self.logger.info("removing client of ID '" + client + "' from room '" + room + "'")
             self.__rooms[room].remove(client)
 
     def close_room(self, room: str):
@@ -570,10 +583,13 @@ class IoTManager(object):
         rooms = self.__rooms.pop(room, None)
 
         if rooms:
+            self.logger.info("closing '" + room + "'")
             for client in rooms:
                 client = self.__clients.get(client, None)
                 if client:
                     client.leave(room, called_by_manager=True)
+        else:
+            self.logger.info("attempted to close room '" + room + "' but it did not exist")
 
     # event decorator function
     def event(self, coroutine):
@@ -587,9 +603,9 @@ class IoTManager(object):
 
         # handle general on_connect, and on_disconnect, and on_update handlers
         if coroutine.__name__ == "on_connect" or coroutine.__name__ == "on_disconnect" or \
-                coroutine.__name__ == "on_update":
+                coroutine.__name__ == "on_update" or coroutine.__name__ == "on_get_offline_validator":
             # logging output
-            self.logger.info("Event handler '" + coroutine.__name__ + "' was added successfully.")
+            self.logger.info("Handler '" + coroutine.__name__ + "' was added successfully.")
 
             # replaces the existing coroutine with the provided one
             setattr(self, coroutine.__name__, coroutine)
@@ -631,6 +647,17 @@ class IoTManager(object):
         """
         pass
 
+    @staticmethod
+    def on_get_offline_validator(client_id: str, endpoint_id: str) \
+            -> Union[ValidationResponse, AbstractEndpointValidator]:
+        """
+
+        :param client_id:
+        :param endpoint_id:
+        :return: Either a ValidationResponse or EndpointValidator object.
+        """
+        return ValidationResponse.CLIENT_NOT_CONNECTED
+
 
 def emit(event: str, data: sendable, client_id: str = None, client_type: str = None, room: str = None,
          trigger_update: bool = True):
@@ -654,7 +681,7 @@ def emit(event: str, data: sendable, client_id: str = None, client_type: str = N
     if client_id is None and client_type is None and room is None:
         client_id = flask.request.g["client"].id
 
-    return flask.current_app.extensions["iot.io"].emit(event, data, client_id, client_type, room)
+    return flask.current_app.extensions["iot.io"].emit(event, data, client_id, client_type, room, trigger_update)
 
 
 def join(room: str, client: Union[IoTClient, str] = None):
